# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

import json
import unittest

from surveyjs import SurveyForm
from tests.utils import load_form, load_creator, readjson, readfile


class TestNestedQuestions(unittest.TestCase):

    def setUp(self):
        raise NotImplementedError("Nested question tests not implemented yet")

if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the ranking question type."""

import unittest

from surveyjs.questions.ranking import QuestionRanking
from tests.utils import load_creator, load_form


class TestQuestionRanking(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['priorityItems']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionRanking)

    def test_type(self):
        self.assertEqual(self.q.type, 'ranking')

    def test_title(self):
        self.assertEqual(self.q.title, 'Prioritize Items')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 4)

    def test_choices_values(self):
        values = self.q.choices_values
        val_names = [v['value'] for v in values]
        self.assertIn('price', val_names)
        self.assertIn('quality', val_names)
        self.assertIn('speed', val_names)
        self.assertIn('support', val_names)

    def test_long_tap(self):
        self.assertTrue(self.q.long_tap)

    def test_select_to_rank_enabled(self):
        self.assertFalse(self.q.select_to_rank_enabled)


class TestRankingForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['priorityItems']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_value_contents(self):
        self.assertEqual(
            self.q.value, ['quality', 'price', 'support', 'speed']
        )

    def test_value_order(self):
        """First item should be highest priority."""
        self.assertEqual(self.q.value[0], 'quality')
        self.assertEqual(self.q.value[-1], 'speed')


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the multipletext question type."""

import unittest

from surveyjs.questions.multipletext import QuestionMultipletext
from tests.utils import load_creator, load_form


class TestQuestionMultipleText(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['address']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionMultipletext)

    def test_type(self):
        self.assertEqual(self.q.type, 'multipletext')

    def test_title(self):
        self.assertEqual(self.q.title, 'Address')

    def test_items(self):
        self.assertEqual(len(self.q.items), 4)

    def test_item_names(self):
        names = self.q.item_names
        self.assertIn('street', names)
        self.assertIn('city', names)
        self.assertIn('state', names)
        self.assertIn('zip', names)

    def test_item_count(self):
        self.assertEqual(self.q.item_count, 4)

    def test_col_count(self):
        self.assertEqual(self.q.col_count, 2)

    def test_get_item_title(self):
        self.assertEqual(self.q.get_item_title('street'), 'Street')
        self.assertEqual(self.q.get_item_title('city'), 'City')


class TestMultipleTextForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['address']

    def test_value_is_dict(self):
        self.assertIsInstance(self.q.value, dict)

    def test_get_item_value(self):
        self.assertEqual(self.q.get_item_value('street'), '123 Main Street')
        self.assertEqual(self.q.get_item_value('city'), 'Springfield')
        self.assertEqual(self.q.get_item_value('state'), 'IL')
        self.assertEqual(self.q.get_item_value('zip'), '62701')

    def test_get_item_value_nonexistent(self):
        self.assertIsNone(self.q.get_item_value('nonexistent'))


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the dropdown question type."""

import unittest

from surveyjs.questions.dropdown import QuestionDropdown
from tests.utils import load_creator, load_form


class TestQuestionDropdown(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['country']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionDropdown)

    def test_type(self):
        self.assertEqual(self.q.type, 'dropdown')

    def test_title(self):
        self.assertEqual(self.q.title, 'Country')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 4)

    def test_choices_values(self):
        values = self.q.choices_values
        val_names = [v['value'] for v in values]
        self.assertIn('us', val_names)
        self.assertIn('uk', val_names)

    def test_has_other(self):
        self.assertFalse(self.q.has_other)

    def test_allow_clear(self):
        self.assertTrue(self.q.allow_clear)

    def test_search_enabled(self):
        self.assertTrue(self.q.search_enabled)


class TestDropdownForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['country']

    def test_value(self):
        self.assertEqual(self.q.value, 'us')

    def test_value_text(self):
        self.assertEqual(self.q.value_text, 'United States')


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the html question type."""

import unittest

from surveyjs.questions.html import QuestionHtml
from tests.utils import load_creator, load_form


class TestQuestionHtml(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['infoHtml']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionHtml)

    def test_type(self):
        self.assertEqual(self.q.type, 'html')

    def test_is_input(self):
        self.assertFalse(self.q.is_input)

    def test_html(self):
        self.assertIn('Thank you', self.q.html)
        self.assertIn('<p>', self.q.html)

    def test_not_in_input_questions(self):
        self.assertNotIn('infoHtml', self.survey.input_questions)

    def test_in_questions(self):
        self.assertIn('infoHtml', self.survey.questions)


class TestHtmlForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_not_in_input_questions(self):
        self.assertNotIn('infoHtml', self.form.input_questions)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the paneldynamic question type."""

import unittest

from surveyjs.questions.paneldynamic import QuestionPaneldynamic
from tests.utils import load_creator, load_form


class TestQuestionPaneldynamic(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['education']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionPaneldynamic)

    def test_type(self):
        self.assertEqual(self.q.type, 'paneldynamic')

    def test_title(self):
        self.assertEqual(self.q.title, 'Education History')

    def test_template_elements(self):
        self.assertEqual(len(self.q.template_elements), 3)

    def test_panel_count(self):
        self.assertEqual(self.q.panel_count, 1)

    def test_min_panel_count(self):
        self.assertEqual(self.q.min_panel_count, 0)

    def test_max_panel_count(self):
        self.assertEqual(self.q.max_panel_count, 5)

    def test_panel_add_text(self):
        self.assertEqual(self.q.panel_add_text, 'Add Education')

    def test_panel_remove_text(self):
        self.assertEqual(self.q.panel_remove_text, 'Remove')

    def test_allow_add_panel(self):
        self.assertTrue(self.q.allow_add_panel)

    def test_allow_remove_panel(self):
        self.assertTrue(self.q.allow_remove_panel)

    def test_template_title(self):
        self.assertEqual(self.q.template_title, 'Education #{panelIndex}')

    def test_is_input(self):
        """paneldynamic is an input element (its value is a list)."""
        self.assertTrue(self.q.is_input)


class TestPanelDynamicForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['education']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_panels_data(self):
        data = self.q.panels_data
        self.assertEqual(len(data), 2)

    def test_actual_panel_count(self):
        self.assertEqual(self.q.actual_panel_count, 2)

    def test_get_panel_value(self):
        self.assertEqual(self.q.get_panel_value(0, 'institution'), 'MIT')
        self.assertEqual(
            self.q.get_panel_value(0, 'degree'), 'BSc Computer Science'
        )
        self.assertEqual(self.q.get_panel_value(0, 'year'), 2015)

    def test_get_panel_value_second(self):
        self.assertEqual(self.q.get_panel_value(1, 'institution'), 'Stanford')
        self.assertEqual(self.q.get_panel_value(1, 'degree'), 'MSc AI')
        self.assertEqual(self.q.get_panel_value(1, 'year'), 2017)

    def test_get_panel_value_out_of_range(self):
        self.assertIsNone(self.q.get_panel_value(99, 'institution'))

    def test_get_panel_value_nonexistent_field(self):
        self.assertIsNone(self.q.get_panel_value(0, 'nonexistent'))


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionHtml(Question):
    """SurveyJS HTML question.

    Displays static HTML content. Not an input element.
    """

    @property
    def is_input(self):
        """HTML elements are display-only, not input questions."""
        return False

    @property
    def html(self):
        """Get the HTML content."""
        return self.raw.get('html', '')



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionMultipletext(Question):
    """SurveyJS Multiple Textboxes question.

    Value is a dict mapping item names to their values.
    Example: {"firstName": "John", "lastName": "Doe"}
    """

    @property
    def items(self):
        """Get the list of text items definitions."""
        return self.raw.get('items', [])

    @property
    def item_names(self):
        """Get the list of item names."""
        return [item.get('name', '') for item in self.items]

    @property
    def item_count(self):
        return len(self.items)

    @property
    def col_count(self):
        return self.raw.get('colCount', 1)

    def get_item_value(self, item_name):
        """Get the value for a specific item by name."""
        val = self.value
        if val and isinstance(val, dict):
            return val.get(item_name)
        return None

    def get_item_title(self, item_name):
        """Get the title for a specific item by name."""
        for item in self.items:
            if item.get('name') == item_name:
                return item.get('title', item_name)
        return item_name



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

import json
import uuid
from collections import OrderedDict


class Question:
    """
    Base class for all SurveyJS question types.

    Holds the raw JSON element data, provides access to common properties
    (name, type, title, etc.) and handles value loading from form data.
    """

    _none_value = None

    def __init__(self, raw, survey, **kwargs):
        """
        @param raw: The raw JSON element dict from the survey schema
        @param survey: The Survey instance that owns this question
        """
        self.raw = raw
        self.survey = survey

        self._parent = None
        self._question_owner = None

        # Child questions (for panels and other containers)
        self.questions = OrderedDict()

        # Path tracking
        self.survey_path = []
        self.survey_input_path = []

        # ID
        self._id = self.raw.get('id', str(uuid.uuid4()))

        # Submission data at this level
        self.form = {}
        self._all_data = {}

        # i18n
        self.language = kwargs.get('language', 'en')
        self.i18n = kwargs.get('i18n', {})

        self.default_value = self.raw.get('defaultValue')

    def load(self, question_owner, parent=None, data=None, is_form=False):
        """Load the question into its owner (Survey or Form) and populate
        data from form submission if applicable."""
        self.question_owner = question_owner

        if parent:
            self.parent = parent

        self._all_data = data or {}
        self.load_data(data, is_form=is_form)

    def load_data(self, data, is_form=False):
        """Load value from form submission data."""
        if self.is_input and data:
            try:
                self.value = data[self.name]
            except KeyError:
                # Use default value if available
                pass

    # --- Properties ---

    @property
    def id(self):
        return self._id

    @property
    def name(self):
        """The unique name/key of the question."""
        return self.raw.get('name', '')

    @property
    def type(self):
        """The SurveyJS question type."""
        return self.raw.get('type', '')

    @property
    def title(self):
        """The display title of the question."""
        title = self.raw.get('title')
        if not title:
            title = self.name
        if self.i18n.get(self.language):
            return self.i18n[self.language].get(title, title)
        return title

    @property
    def label(self):
        """Alias for title."""
        return self.title

    @property
    def description(self):
        """The question description/help text."""
        return self.raw.get('description', '')

    @property
    def is_required(self):
        """Whether the question is required."""
        return self.raw.get('isRequired', False)

    @property
    def is_input(self):
        """Whether this is an input question (has user-submitted values).
        Layout elements (panel, html, image) override this to return False."""
        return True

    @property
    def is_visible(self):
        """Whether the question is visible."""
        return self.raw.get('visible', True)

    @property
    def visible_if(self):
        """The visibleIf expression."""
        return self.raw.get('visibleIf', '')

    @property
    def enable_if(self):
        """The enableIf expression."""
        return self.raw.get('enableIf', '')

    @property
    def read_only(self):
        """Whether the question is read-only."""
        return self.raw.get('readOnly', False)

    @property
    def validators(self):
        """The list of validators."""
        return self.raw.get('validators', [])

    @property
    def input_type(self):
        """The input type (for text questions)."""
        return self.raw.get('inputType', '')

    @property
    def placeholder(self):
        """The placeholder text."""
        return self.raw.get('placeholder', '')

    @property
    def parent(self):
        return self._parent

    @parent.setter
    def parent(self, parent):
        if parent:
            self._parent = parent
            self._parent.questions[self.name] = self

    @property
    def question_owner(self):
        """The question's owner (Survey or Form)."""
        return self._question_owner

    @question_owner.setter
    def question_owner(self, question_owner):
        self._question_owner = question_owner
        if self.is_input:
            self._question_owner.input_questions[self.name] = self

    # --- Value ---

    @property
    def value(self):
        """Get the question's value from form data."""
        val = self.form.get('value', self._none_value)
        if val is self._none_value:
            return self.default_value
        return val

    @value.setter
    def value(self, value):
        self.form['value'] = value

    @property
    def raw_value(self):
        """Get the raw (unprocessed) value."""
        return self.form.get('raw_value', self.value)

    @raw_value.setter
    def raw_value(self, value):
        self.form['raw_value'] = value

    # --- Representation ---

    def __repr__(self):
        return '<%s name=%s>' % (self.__class__.__name__, self.name)

    def to_dict(self):
        """Return a dictionary representation of the question."""
        return {
            'name': self.name,
            'type': self.type,
            'title': self.title,
            'value': self.value,
        }



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the matrix question type."""

import unittest

from surveyjs.questions.matrix import QuestionMatrix
from tests.utils import load_creator, load_form


class TestQuestionMatrix(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['qualityMatrix']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionMatrix)

    def test_type(self):
        self.assertEqual(self.q.type, 'matrix')

    def test_title(self):
        self.assertEqual(self.q.title, 'Rate the quality')

    def test_columns(self):
        self.assertEqual(len(self.q.columns), 4)

    def test_column_values(self):
        values = self.q.column_values
        val_names = [v['value'] for v in values]
        self.assertIn('poor', val_names)
        self.assertIn('excellent', val_names)

    def test_rows(self):
        self.assertEqual(len(self.q.rows), 3)

    def test_row_values(self):
        values = self.q.row_values
        val_names = [v['value'] for v in values]
        self.assertIn('product', val_names)
        self.assertIn('service', val_names)
        self.assertIn('support', val_names)

    def test_is_all_row_required(self):
        self.assertTrue(self.q.is_all_row_required)


class TestMatrixForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['qualityMatrix']

    def test_value_is_dict(self):
        self.assertIsInstance(self.q.value, dict)

    def test_get_row_value(self):
        self.assertEqual(self.q.get_row_value('product'), 'excellent')
        self.assertEqual(self.q.get_row_value('service'), 'good')
        self.assertEqual(self.q.get_row_value('support'), 'fair')

    def test_get_row_value_nonexistent(self):
        self.assertIsNone(self.q.get_row_value('nonexistent'))


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the panel question type."""

import unittest

from surveyjs.questions.panel import QuestionPanel
from tests.utils import load_creator, load_form


class TestQuestionPanel(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['contactPanel']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionPanel)

    def test_type(self):
        self.assertEqual(self.q.type, 'panel')

    def test_panel_title(self):
        self.assertEqual(self.q.panel_title, 'Contact Information')

    def test_is_input(self):
        self.assertFalse(self.q.is_input)

    def test_state(self):
        self.assertEqual(self.q.state, 'expanded')

    def test_inner_indent(self):
        self.assertEqual(self.q.inner_indent, 1)

    def test_elements(self):
        self.assertEqual(len(self.q.elements), 2)

    def test_not_in_input_questions(self):
        self.assertNotIn('contactPanel', self.survey.input_questions)

    def test_in_questions(self):
        self.assertIn('contactPanel', self.survey.questions)

    def test_nested_questions_loaded(self):
        """Questions inside the panel should be loaded into the survey."""
        self.assertIn('phone', self.survey.questions)
        self.assertIn('website', self.survey.questions)


class TestPanelForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_not_in_input_questions(self):
        self.assertNotIn('contactPanel', self.form.input_questions)

    def test_nested_values_accessible(self):
        """Questions inside panel should get values from the flat form data."""
        self.assertEqual(self.form.get_value('phone'), '+1-555-0123')
        self.assertEqual(
            self.form.get_value('website'), 'https://alice.example.com'
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the tagbox question type."""

import unittest

from surveyjs.questions.tagbox import QuestionTagbox
from tests.utils import load_creator, load_form


class TestQuestionTagbox(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['skills']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionTagbox)

    def test_type(self):
        self.assertEqual(self.q.type, 'tagbox')

    def test_title(self):
        self.assertEqual(self.q.title, 'Skills')

    def test_choices(self):
        self.assertEqual(len(self.q.choices), 5)

    def test_choices_values(self):
        values = self.q.choices_values
        val_names = [v['value'] for v in values]
        self.assertIn('python', val_names)
        self.assertIn('javascript', val_names)

    def test_has_other(self):
        self.assertTrue(self.q.has_other)

    def test_search_enabled(self):
        self.assertTrue(self.q.search_enabled)

    def test_max_selected_choices(self):
        self.assertEqual(self.q.max_selected_choices, 4)


class TestTagboxForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['skills']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_value_contents(self):
        self.assertEqual(self.q.value, ['python', 'javascript', 'go'])

    def test_value_texts(self):
        texts = self.q.value_texts
        self.assertIn('Python', texts)
        self.assertIn('JavaScript', texts)
        self.assertIn('Go', texts)

    def test_value_texts_count(self):
        self.assertEqual(len(self.q.value_texts), 3)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from datetime import date, datetime

from .question import Question


class QuestionNonValue(Question):
    """SurveyJS Text (Single-Line Input) question.

    Supports inputType variants: text, number, date, datetime-local,
    email, url, password, tel, color, range, time, month, week.
    """

    @property
    def max_length(self):
        return self.raw.get('maxLength', 0)

    @property
    def min(self):
        return self.raw.get('min')

    @property
    def max(self):
        return self.raw.get('max')

    @property
    def step(self):
        return self.raw.get('step')

    @property
    def input_type(self):
        return self.raw.get('inputType', 'text')

    def to_number(self):
        """Convert value to a number (int or float).
        Applicable when inputType is 'number' or 'range'."""
        val = self.value
        if val is None:
            return None
        try:
            if isinstance(val, (int, float)):
                return val
            if '.' in str(val):
                return float(val)
            return int(val)
        except (ValueError, TypeError):
            return None

    def to_date(self):
        """Convert value to a date object.
        Applicable when inputType is 'date'."""
        val = self.value
        if val is None:
            return None
        if isinstance(val, date):
            return val
        try:
            return datetime.strptime(str(val), '%Y-%m-%d').date()
        except (ValueError, TypeError):
            return None

    def to_datetime(self):
        """Convert value to a datetime object.
        Applicable when inputType is 'datetime-local'."""
        val = self.value
        if val is None:
            return None
        if isinstance(val, datetime):
            return val
        for fmt in ('%Y-%m-%dT%H:%M:%S', '%Y-%m-%dT%H:%M', '%Y-%m-%d %H:%M:%S'):
            try:
                return datetime.strptime(str(val), fmt)
            except ValueError:
                continue
        return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionRating(Question):
    """SurveyJS Rating Scale question.

    Value is typically a number or string from the rate values.
    """

    @property
    def rate_min(self):
        return self.raw.get('rateMin', 1)

    @property
    def rate_max(self):
        return self.raw.get('rateMax', 5)

    @property
    def rate_step(self):
        return self.raw.get('rateStep', 1)

    @property
    def rate_values(self):
        """Custom rate values, if specified."""
        return self.raw.get('rateValues', [])

    @property
    def rate_count(self):
        return self.raw.get('rateCount', 5)

    @property
    def min_rate_description(self):
        return self.raw.get('minRateDescription', '')

    @property
    def max_rate_description(self):
        return self.raw.get('maxRateDescription', '')

    @property
    def display_mode(self):
        """'auto', 'buttons', or 'dropdown'."""
        return self.raw.get('displayMode', 'auto')

    @property
    def rate_type(self):
        """'labels', 'stars', or 'smileys'."""
        return self.raw.get('rateType', 'labels')

    def to_number(self):
        """Convert value to a number."""
        val = self.value
        if val is None:
            return None
        try:
            if isinstance(val, (int, float)):
                return val
            return int(val)
        except (ValueError, TypeError):
            try:
                return float(val)
            except (ValueError, TypeError):
                return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the signaturepad question type."""

import unittest

from surveyjs.questions.signaturepad import QuestionSignaturepad
from tests.utils import load_creator, load_form


class TestQuestionSignaturepad(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['signature']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionSignaturepad)

    def test_type(self):
        self.assertEqual(self.q.type, 'signaturepad')

    def test_title(self):
        self.assertEqual(self.q.title, 'Your Signature')

    def test_signature_width(self):
        self.assertEqual(self.q.signature_width, 500)

    def test_signature_height(self):
        self.assertEqual(self.q.signature_height, 200)

    def test_pen_color(self):
        self.assertEqual(self.q.pen_color, '#000000')

    def test_background_color(self):
        self.assertEqual(self.q.background_color, '#ffffff')

    def test_data_format(self):
        self.assertEqual(self.q.data_format, 'png')

    def test_allow_clear(self):
        self.assertTrue(self.q.allow_clear)


class TestSignaturepadForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['signature']

    def test_value_is_base64(self):
        self.assertTrue(self.q.value.startswith('data:image/png;base64,'))

    def test_is_base64(self):
        self.assertTrue(self.q.is_base64)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the Survey class."""

import unittest

from tests.utils import readjson, load_creator


class TestSurvey(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()

    def test_title(self):
        self.assertEqual(self.survey.title, 'Test Survey')

    def test_description(self):
        self.assertIn('comprehensive', self.survey.description)

    def test_pages(self):
        self.assertEqual(len(self.survey.pages), 5)

    def test_pages_names(self):
        names = [p['name'] for p in self.survey.pages]
        self.assertIn('page_basic_input', names)
        self.assertIn('page_choice', names)
        self.assertIn('page_special', names)
        self.assertIn('page_matrix', names)
        self.assertIn('page_layout', names)

    def test_questions_loaded(self):
        """All top-level and nested questions should be loaded."""
        self.assertGreater(len(self.survey.questions), 0)

    def test_input_questions(self):
        """Input questions should not include panel, html, image."""
        for name, q in self.survey.input_questions.items():
            self.assertTrue(q.is_input, f'{name} should be an input question')

    def test_non_input_questions_excluded(self):
        """Panel, html, image should not be in input_questions."""
        non_input = ['contactPanel', 'infoHtml', 'thankYouImage']
        for name in non_input:
            self.assertNotIn(name, self.survey.input_questions)

    def test_non_input_questions_in_questions(self):
        """Panel, html, image should still be in questions dict."""
        non_input = ['contactPanel', 'infoHtml', 'thankYouImage']
        for name in non_input:
            self.assertIn(name, self.survey.questions)

    def test_question_types(self):
        """Verify some expected question types."""
        self.assertEqual(self.survey.questions['firstName'].type, 'text')
        self.assertEqual(self.survey.questions['bio'].type, 'comment')
        self.assertEqual(self.survey.questions['favoriteColor'].type, 'radiogroup')
        self.assertEqual(self.survey.questions['hobbies'].type, 'checkbox')
        self.assertEqual(self.survey.questions['country'].type, 'dropdown')
        self.assertEqual(self.survey.questions['agreeTerms'].type, 'boolean')

    def test_nested_panel_questions(self):
        """Questions inside panels should be loaded."""
        self.assertIn('phone', self.survey.questions)
        self.assertIn('website', self.survey.questions)

    def test_survey_from_json_string(self):
        """Survey should accept a JSON string."""
        import json
        schema = readjson('test_survey_schema.json')
        from surveyjs import SurveyCreator
        creator = SurveyCreator(json.dumps(schema))
        self.assertEqual(creator.title, 'Test Survey')

    def test_get_question_class_unknown(self):
        """Unknown question types should fall back to base Question."""
        from surveyjs.questions.question import Question
        cls = self.survey.get_question_class({'type': 'unknowntype'})
        self.assertEqual(cls, Question)

    def test_form_property(self):
        """Survey's form property should be an empty dict."""
        self.assertEqual(self.survey.form, {})


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionImagepicker(Question):
    """SurveyJS Image Picker question.

    Value is the selected image's value, or a list for multi-select.
    """

    @property
    def choices(self):
        return self.raw.get('choices', [])

    @property
    def multi_select(self):
        return self.raw.get('multiSelect', False)

    @property
    def show_label(self):
        return self.raw.get('showLabel', False)

    @property
    def image_fit(self):
        return self.raw.get('imageFit', 'contain')

    @property
    def image_height(self):
        return self.raw.get('imageHeight', 150)

    @property
    def image_width(self):
        return self.raw.get('imageWidth', 200)

    @property
    def content_mode(self):
        """'image' or 'video'."""
        return self.raw.get('contentMode', 'image')



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the text question type."""

import unittest
from datetime import date, datetime

from surveyjs.questions.text import QuestionText
from tests.utils import load_creator, load_form


class TestQuestionText(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()

    def test_class_type(self):
        q = self.survey.questions['firstName']
        self.assertIsInstance(q, QuestionText)

    def test_name(self):
        self.assertEqual(self.survey.questions['firstName'].name, 'firstName')

    def test_title(self):
        self.assertEqual(self.survey.questions['firstName'].title, 'First Name')

    def test_type(self):
        self.assertEqual(self.survey.questions['firstName'].type, 'text')

    def test_description(self):
        self.assertEqual(
            self.survey.questions['firstName'].description,
            'Enter your first name'
        )

    def test_is_required(self):
        self.assertTrue(self.survey.questions['firstName'].is_required)

    def test_placeholder(self):
        self.assertEqual(self.survey.questions['firstName'].placeholder, 'John')

    def test_input_type_text(self):
        self.assertEqual(self.survey.questions['firstName'].input_type, 'text')

    def test_input_type_number(self):
        self.assertEqual(self.survey.questions['age'].input_type, 'number')

    def test_input_type_date(self):
        self.assertEqual(self.survey.questions['birthDate'].input_type, 'date')

    def test_input_type_email(self):
        self.assertEqual(self.survey.questions['email'].input_type, 'email')

    def test_max_length(self):
        self.assertEqual(self.survey.questions['email'].max_length, 100)

    def test_min_max_step(self):
        q = self.survey.questions['age']
        self.assertEqual(q.min, 0)
        self.assertEqual(q.max, 150)
        self.assertEqual(q.step, 1)

    def test_validators(self):
        q = self.survey.questions['email']
        self.assertEqual(len(q.validators), 1)
        self.assertEqual(q.validators[0]['type'], 'email')

    def test_is_input(self):
        self.assertTrue(self.survey.questions['firstName'].is_input)

    def test_default_value_none(self):
        """Survey-level text questions normally have no value."""
        self.assertIsNone(self.survey.questions['firstName'].value)


class TestTextForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_value_string(self):
        self.assertEqual(self.form.input_questions['firstName'].value, 'Alice')

    def test_value_number(self):
        self.assertEqual(self.form.input_questions['age'].value, 30)

    def test_value_email(self):
        self.assertEqual(
            self.form.input_questions['email'].value, 'alice@example.com'
        )

    def test_to_number(self):
        q = self.form.input_questions['age']
        self.assertEqual(q.to_number(), 30)

    def test_to_number_none(self):
        """to_number on a text (non-number) field returns None if not numeric."""
        q = self.form.input_questions['firstName']
        self.assertIsNone(q.to_number())

    def test_to_date(self):
        q = self.form.input_questions['birthDate']
        result = q.to_date()
        self.assertIsInstance(result, date)
        self.assertEqual(result, date(1993, 5, 15))

    def test_to_datetime(self):
        q = self.form.input_questions['appointmentTime']
        result = q.to_datetime()
        self.assertIsInstance(result, datetime)
        self.assertEqual(result, datetime(2024, 6, 15, 10, 30))

    def test_to_dict(self):
        q = self.form.input_questions['firstName']
        d = q.to_dict()
        self.assertEqual(d['name'], 'firstName')
        self.assertEqual(d['value'], 'Alice')
        self.assertEqual(d['type'], 'text')


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the rating question type."""

import unittest

from surveyjs.questions.rating import QuestionRating
from tests.utils import load_creator, load_form


class TestQuestionRating(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['satisfaction']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionRating)

    def test_type(self):
        self.assertEqual(self.q.type, 'rating')

    def test_title(self):
        self.assertEqual(self.q.title, 'Overall Satisfaction')

    def test_rate_min(self):
        self.assertEqual(self.q.rate_min, 1)

    def test_rate_max(self):
        self.assertEqual(self.q.rate_max, 10)

    def test_rate_step(self):
        self.assertEqual(self.q.rate_step, 1)

    def test_min_rate_description(self):
        self.assertEqual(self.q.min_rate_description, 'Very Bad')

    def test_max_rate_description(self):
        self.assertEqual(self.q.max_rate_description, 'Excellent')

    def test_display_mode(self):
        self.assertEqual(self.q.display_mode, 'buttons')

    def test_rate_type(self):
        self.assertEqual(self.q.rate_type, 'stars')


class TestRatingForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['satisfaction']

    def test_value(self):
        self.assertEqual(self.q.value, 8)

    def test_to_number(self):
        self.assertEqual(self.q.to_number(), 8)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionRanking(Question):
    """SurveyJS Ranking question.

    Value is a list of choices in the user's ranked order.
    """

    @property
    def choices(self):
        return self.raw.get('choices', [])

    @property
    def choices_values(self):
        result = []
        for choice in self.choices:
            if isinstance(choice, dict):
                result.append({
                    'value': choice.get('value'),
                    'text': choice.get('text', str(choice.get('value', '')))
                })
            else:
                result.append({'value': choice, 'text': str(choice)})
        return result

    @property
    def long_tap(self):
        return self.raw.get('longTap', True)

    @property
    def select_to_rank_enabled(self):
        return self.raw.get('selectToRankEnabled', False)



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

import json
import logging
from collections import OrderedDict
from copy import deepcopy

from surveyjs.questions.question import Question

logger = logging.getLogger(__name__)


class SurveyCreator:
    """
    Represents a SurveyJS Creator schema (the survey blueprint/design).

    Parses the survey JSON schema and creates Question objects for each
    question element found in the schema.
    """

    def __init__(
        self,
        schema_json,
        language='en',
        i18n=None,
        question_class_mapping={},
        **kwargs
    ):
        """
        @param schema_json: SurveyJS Creator JSON schema (str or dict)
        @param language: Language code for translations (default: 'en')
        @param i18n: Translations dict
        """
        if isinstance(schema_json, dict):
            self.schema = schema_json
        else:
            self.schema = json.loads(schema_json)

        self.language = language
        self.i18n = i18n or {}

        self.question_class_mapping = question_class_mapping

        # All questions (input + layout) keyed by name
        self.questions = OrderedDict()

        # Input-only questions keyed by name (no panels, html, image)
        self.input_questions = OrderedDict()

        # Questions keyed by internal id
        self.question_ids = OrderedDict()

        # Load all questions from the schema
        self.load_questions()

    @property
    def title(self):
        return self.schema.get('title', '')

    @property
    def description(self):
        return self.schema.get('description', '')

    @property
    def pages(self):
        return self.schema.get('pages', [])

    @property
    def components(self):
        return self.questions

    @property
    def input_components(self):
        return self.input_questions

    def load_questions(self):
        """Load questions from the schema, handling both pages-based and
        flat elements-based schemas."""
        pages = self.schema.get('pages')
        if pages:
            for page in pages:
                elements = page.get('elements', [])
                self._load_elements(elements)
        else:
            # Single page: elements at top level
            elements = self.schema.get('elements', [])
            self._load_elements(elements)

    def _load_elements(self, elements, parent=None):
        """Recursively load elements from the schema."""
        for element in elements:
            if 'type' not in element:
                continue

            question_obj = self.get_question_object(element)
            if not question_obj:
                continue

            question_obj.load(
                question_owner=self,
                parent=parent,
                data=None,
                is_form=False,
            )
            self.questions[question_obj.name] = question_obj

            if question_obj.id:
                self.question_ids[question_obj.id] = question_obj

            # Recurse into panel/page elements
            if element['type'] in ('panel', 'paneldynamic'):
                nested = element.get('elements', [])
                self._load_elements(nested, parent=question_obj)

    def get_question_class(self, element):
        """Dynamically load the question class based on the element type."""
        element_type = element.get('type')
        element_type_cap = element_type.capitalize() if element_type else None
        if not element_type:
            return None

        cls_mapping = self.question_class_mapping.get(element_type)
        if cls_mapping:
            cls_mapping = cls_mapping.capitalize() if isinstance(cls_mapping, str) else cls_mapping
            if isinstance(cls_mapping, str):
                cls_name = f"Question{cls_mapping}"
                import_path = f"surveyjs.questions.{element_type}.{cls_mapping}"
                try:
                    module = __import__(import_path, fromlist=[cls_name])
                    cls = getattr(module, cls_name)
                    return cls
                except (AttributeError, ModuleNotFoundError) as e:
                    logger.warning(
                        "Could not load question class for type '%s': %s. "
                        "Falling back to base Question.", element_type, e
                    )
                    return Question
            else:
                return cls_mapping
        else:
            cls_name = f"Question{element_type_cap}"
            import_path = 'surveyjs.questions.%s' % element_type
            try:
                module = __import__(import_path, fromlist=[cls_name])
                return getattr(module, cls_name)
            except (AttributeError, ModuleNotFoundError) as e:
                logger.warning(
                    "Could not load question class for type '%s': %s. "
                    "Falling back to base Question.", element_type, e
                )
                return Question

    def get_question_object(self, element):
        """Create a question object from an element dict."""
        cls = self.get_question_class(element)
        if cls is None:
            return None
        return cls(element, self, language=self.language, i18n=self.i18n)

    @property
    def form(self):
        """Placeholder form dict (always empty). Useful in contexts where
        a question owner's form is requested."""
        return {}



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the matrixdynamic question type."""

import unittest

from surveyjs.questions.matrixdynamic import QuestionMatrixdynamic
from tests.utils import load_creator, load_form


class TestQuestionMatrixDynamic(unittest.TestCase):

    def setUp(self):
        self.survey = load_creator()
        self.q = self.survey.questions['employeeList']

    def test_class_type(self):
        self.assertIsInstance(self.q, QuestionMatrixdynamic)

    def test_type(self):
        self.assertEqual(self.q.type, 'matrixdynamic')

    def test_title(self):
        self.assertEqual(self.q.title, 'Employee List')

    def test_columns(self):
        self.assertEqual(len(self.q.columns), 3)

    def test_column_names(self):
        col_names = [c['name'] for c in self.q.columns]
        self.assertIn('name', col_names)
        self.assertIn('role', col_names)
        self.assertIn('salary', col_names)

    def test_row_count(self):
        self.assertEqual(self.q.row_count, 2)

    def test_min_row_count(self):
        self.assertEqual(self.q.min_row_count, 1)

    def test_max_row_count(self):
        self.assertEqual(self.q.max_row_count, 10)

    def test_cell_type(self):
        self.assertEqual(self.q.cell_type, 'text')


class TestMatrixDynamicForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()
        self.q = self.form.input_questions['employeeList']

    def test_value_is_list(self):
        self.assertIsInstance(self.q.value, list)

    def test_rows_data(self):
        self.assertEqual(len(self.q.rows_data), 3)

    def test_actual_row_count(self):
        self.assertEqual(self.q.actual_row_count, 3)

    def test_get_cell_value(self):
        self.assertEqual(self.q.get_cell_value(0, 'name'), 'Bob Smith')
        self.assertEqual(self.q.get_cell_value(0, 'role'), 'developer')
        self.assertEqual(self.q.get_cell_value(1, 'name'), 'Jane Doe')
        self.assertEqual(self.q.get_cell_value(2, 'salary'), '110000')

    def test_get_cell_value_out_of_range(self):
        self.assertIsNone(self.q.get_cell_value(99, 'name'))


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

"""Tests for the Form class."""

import json
import unittest

from surveyjs import SurveyForm
from tests.utils import load_form, load_creator, readjson, readfile


class TestForm(unittest.TestCase):

    def setUp(self):
        self.form = load_form()

    def test_form_questions_loaded(self):
        self.assertGreater(len(self.form.questions), 0)

    def test_form_input_questions(self):
        self.assertGreater(len(self.form.input_questions), 0)

    def test_form_data_accessor(self):
        """form.data.firstName should return the question object."""
        q = self.form.data.firstName
        self.assertIsNotNone(q)
        self.assertEqual(q.value, 'Alice')

    def test_form_data_accessor_nonexistent(self):
        """Accessing a nonexistent question via data returns None."""
        self.assertIsNone(self.form.data.nonExistentQuestion)

    def test_get_question_by_name(self):
        q = self.form.get_question_by_name('firstName')
        self.assertIsNotNone(q)
        self.assertEqual(q.name, 'firstName')

    def test_get_value(self):
        self.assertEqual(self.form.get_value('firstName'), 'Alice')
        self.assertEqual(self.form.get_value('age'), 30)

    def test_get_value_nonexistent(self):
        self.assertIsNone(self.form.get_value('nonExistent'))

    def test_form_from_survey_schema_json(self):
        """Form can be created with creator_schema_json instead of creator."""
        schema = readfile('test_survey_schema.json')
        form_json = readjson('test_survey_form.json')
        form = SurveyForm(form_json, creator_schema_json=schema)
        self.assertEqual(form.get_value('firstName'), 'Alice')

    def test_form_constructor_both_raises(self):
        """Providing both survey and survey_schema_json should raise."""
        schema = readjson('test_survey_schema.json')
        form_data = readjson('test_survey_form.json')
        creator = load_creator()
        with self.assertRaises(Exception):
            SurveyForm(form_data, creator=creator, creator_schema_json=schema)

    def test_form_constructor_neither_raises(self):
        """Providing neither survey nor survey_schema_json should raise."""
        form_data = readjson('test_survey_form.json')
        with self.assertRaises(Exception):
            SurveyForm(form_data)

    def test_form_from_json_string(self):
        """Form should accept a JSON string for form_json."""

        form_json = readjson('test_survey_form.json')
        creator = load_creator()
        form = SurveyForm(json.dumps(form_json), creator=creator)
        self.assertEqual(form.get_value('firstName'), 'Alice')

    def test_form_all_input_values_loaded(self):
        """All submitted values should be accessible."""
        expected = {
            'firstName': 'Alice',
            'age': 30,
            'country': 'us',
            'satisfaction': 8,
        }
        for name, expected_val in expected.items():
            self.assertEqual(
                self.form.get_value(name), expected_val,
                f"Value mismatch for {name}"
            )

    def test_non_input_not_in_input_questions(self):
        """Panel, html, image should not be in input_questions."""
        non_input = ['contactPanel', 'infoHtml', 'thankYouImage']
        for name in non_input:
            self.assertNotIn(name, self.form.input_questions)


if __name__ == '__main__':
    unittest.main()



# Copyright 2026 Nova Code (https://www.novacode.nl)
# See LICENSE file for full licensing details.

from .question import Question


class QuestionExpression(Question):
    """SurveyJS Expression question.

    A read-only calculated field. Value is computed from an expression.
    """

    @property
    def expression(self):
        """The calculation expression."""
        return self.raw.get('expression', '')

    @property
    def display_style(self):
        """'none', 'decimal', 'currency', or 'percent'."""
        return self.raw.get('displayStyle', 'none')

    @property
    def currency(self):
        return self.raw.get('currency', 'USD')

    @property
    def maximum_fraction_digits(self):
        return self.raw.get('maximumFractionDigits', -1)

    @property
    def minimum_fraction_digits(self):
        return self.raw.get('minimumFractionDigits', -1)

    @property
    def use_grouping(self):
        return self.raw.get('useGrouping', True)

    def to_number(self):
        """Try to convert the expression value to a number."""
        val = self.value
        if val is None:
            return None
        try:
            if isinstance(val, (int, float)):
                return val
            return float(val)
        except (ValueError, TypeError):
            return None



# Copyright 2026 Nova Code (https://www.novacode.nl)
"""
SurveyJS Python Package

This module serves as the entry point for the surveyjs package, providing
access to the core components for working with SurveyJS forms and the
survey creator.

It imports and exposes the following classes:
    - SurveyForm: Handles the rendering and processing of SurveyJS forms.
    - SurveyCreator: Provides functionality for creating and managing surveys.

Copyright 2026 Nova Code (https://www.novacode.nl)
See LICENSE file for full licensing details.
"""
# See LICENSE file for full licensing details.

from surveyjs.creator import SurveyCreator
from surveyjs.form import SurveyForm

__all__ = ['SurveyCreator', 'SurveyForm']
